package com.example.service.graphql;

import com.apollographql.federation.graphqljava.FederationDirectives;
import com.apollographql.federation.graphqljava._FieldSet;
import graphql.schema.*;
import graphql.util.TraversalControl;
import graphql.util.TraverserContext;
import io.quarkus.arc.DefaultBean;

import javax.enterprise.context.ApplicationScoped;
import java.util.Arrays;

import static graphql.util.TraversalControl.CONTINUE;
import static java.util.Objects.requireNonNull;
import static java.util.stream.Collectors.joining;

/**
 * {@inheritDoc}
 * <p>
 * GraphQL Visitor to modify federation directives in {@link GraphQLSchema}
 * </p>
 */
@DefaultBean
@ApplicationScoped
public class FederationDirectiveVisitor extends GraphQLTypeVisitorStub {

    /**
     * {@inheritDoc}
     * <p>
     * Update graphql federation directives to match <i>Apollo</i> contract
     * </p>
     *
     * @param node    {@link GraphQLArgument} node
     * @param context {@link TraverserContext} context
     * @return {@link TraversalControl}
     */
    @SuppressWarnings("deprecation")
    @Override
    public TraversalControl visitGraphQLArgument(GraphQLArgument node, TraverserContext<GraphQLSchemaElement> context) {
        GraphQLSchemaElement parentNode = context.getParentNode();
        if (parentNode instanceof GraphQLDirective directive
                && "fields".equalsIgnoreCase(node.getName())
                && node.getArgumentValue().isSet()
                && (directive.getName().equalsIgnoreCase(FederationDirectives.keyName)
                        || directive.getName().equalsIgnoreCase(FederationDirectives.providesName))) {
            String value = Arrays.stream(((Object[]) requireNonNull(node.getArgumentValue().getValue())))
                    .map(String.class::cast).collect(joining(" "));
            return changeNode(context, GraphQLArgument.newArgument(node).type(_FieldSet.type).value(value).build());
        }
        return CONTINUE;
    }
}
